@@ -155,9 +155,10 @@ class DesignMatrix(object):
     matrix (e.g., no frames had artifacts), these attributes are `None`.
 
     """
-    def __init__(self, design, hrf_model, ntp, regressors=None, confounds=None,
-                 artifacts=None, condition_names=None, confound_pca=False,
-                 tr=2, hpf_cutoff=128, oversampling=16):
+    def __init__(self, design=None, hrf_model=None, ntp=None, regressors=None,
+                 confounds=None, artifacts=None, condition_names=None,
+                 confound_pca=False, tr=2, hpf_cutoff=128, hpf_kernel=None,
+                 oversampling=16):
         """Initialize the design matrix object.
 
         Parameters
@@ -197,19 +198,28 @@ class DesignMatrix(object):
             sampling interval (in seconds) of the data/design
         hpf_cutoff : float
             filter cutoff (in seconds), or None to skip the filter
+        hpf_kernel : ntp x ntp array
+            precomputed matrix to implement the highpass filter; overrides the
+            hpf_cutoff (i.e. it is not checked if they match).
         oversampling : float
             construction of the condition evs and convolution
             are performed on high-resolution data with this oversampling
 
         """
-        if "duration" not in design:
-            design["duration"] = 0
-        if "value" not in design:
-            design["value"] = 1
-
-        self.design = design
         self.tr = tr
+        if hrf_model is None:
+            hrf_model = GammaDifferenceHRF(tr=tr)
 
+        # Try to infer the timepoints from passed regressors or confounds
+        if ntp is None:
+            if regressors is not None:
+                ntp = regressors.shape[0]
+            elif confounds is not None:
+                ntp = confounds.shape[0]
+            else:
+                raise ValueError("Could not infer ntp")
+
+        # Infer the original and resampled frametime vectors
         stop = ntp * tr
         frametimes = np.arange(0, stop, tr, np.float)
         self.frametimes = pd.Series(frametimes, name="frametimes")
@@ -218,22 +228,39 @@ class DesignMatrix(object):
         hires_frametimes = np.arange(0, stop, tr / oversampling, np.float)
         self._hires_frametimes = hires_frametimes
 
-        if condition_names is None:
-            condition_names = np.sort(design.condition.unique())
-        self._condition_names = pd.Series(condition_names, name="conditions")
+        # The design portion is optional
+        if design is None:
+            conditions = None
+            self._condition_names = pd.Series([], name="conditions")
+            pp_heights = []
+            self.design = design
+        else:
+            if "duration" not in design:
+                design["duration"] = 0
+            if "value" not in design:
+                design["value"] = 1
 
-        self._ntp = ntp
+            self.design = design
 
-        # Convolve the oversampled condition evs
-        self._make_hires_base(oversampling)
-        self._convolve(hrf_model)
+            if condition_names is None:
+                condition_names = np.sort(design.condition.unique())
+            self._condition_names = pd.Series(condition_names,
+                                              name="conditions")
 
-        # Subsample the condition evs and highpass filter
-        conditions = self._subsample_condition_matrix()
-        conditions -= conditions.mean()
-        pp_heights = (conditions.max() - conditions.min()).tolist()
-        if hpf_cutoff is not None:
-            conditions = self._highpass_filter(conditions, hpf_cutoff)
+            self._ntp = ntp
+
+            # Convolve the oversampled condition evs
+            self._make_hires_base(oversampling)
+            self._convolve(hrf_model)
+
+            # Subsample the condition evs and highpass filter
+            conditions = self._subsample_condition_matrix()
+            conditions -= conditions.mean()
+            pp_heights = (conditions.max() - conditions.min()).tolist()
+            if hpf_kernel is not None:
+                hpf_cutoff = hpf_kernel
+            if hpf_cutoff is not None:
+                conditions = self._highpass_filter(conditions, hpf_cutoff)
 
         # Set up the other regressors of interest
         regressors = self._validate_component(regressors, "regressor")
@@ -259,7 +286,9 @@ class DesignMatrix(object):
                 artifacts = None
 
         # Now build the full design matrix
-        pieces = [conditions]
+        pieces = []
+        if conditions is not None:
+            pieces.append(conditions)
         if regressors is not None:
             pieces.append(regressors)
         if confounds is not None:
@@ -402,7 +431,10 @@ class DesignMatrix(object):
 
     def _highpass_filter(self, mat, cutoff):
         """Highpass-filter each column in mat."""
-        F = fsl_highpass_matrix(self._ntp, cutoff, self.tr)
+        if np.isscalar(cutoff):
+            F = fsl_highpass_matrix(self._ntp, cutoff, self.tr)
+        else:
+            F = cutoff
         for key, vals in mat.iteritems():
             mat[key] = np.dot(F, vals)
         return mat
@@ -414,17 +446,19 @@ class DesignMatrix(object):
             vector[self.design_matrix.columns == name] = weight
         return vector
 
-    def plot(self, kind="full", fname=None, cmap="bone"):
+    def plot(self, kind="full", fname=None, cmap="bone", close=False):
         """Draw an image representation of the design matrix.
 
         Parameters
         ----------
         kind : string
-            which submatrix to plot, or "full" for the whole matrix
+            Which submatrix to plot, or "full" for the whole matrix.
         fname : string, optional
-            if provided, save the plot to this file name
+            If provided, save the plot to this file name.
         cmap : string or colormap object
-            colormap for the plot
+            Colormap for the plot.
+        close: bool
+            If True, close the figure after plotting.
 
         """
         names = getattr(self, "_%s_names" % kind)
@@ -446,38 +480,37 @@ class DesignMatrix(object):
 
         if fname is not None:
             f.savefig(fname)
+        if close:
+            plt.close(f)
 
-    def plot_confound_correlation(self, fname=None, legend=True):
+    def plot_confound_correlation(self, fname=None, legend=True, close=False):
         """Plot how correlated the condition and confound regressors are."""
         import seaborn as sns
         corrs = self.design_matrix.corr()
         corrs = corrs.loc[self._confound_names, self._condition_names]
 
         n_bars = len(self._condition_names) * len(self._confound_names)
-        xsize = min(n_bars * .2, 10)
-        figsize = (xsize, 4)
+        ysize = min(n_bars * .2, 10)
+        figsize = (9, ysize)
 
         f, ax = plt.subplots(1, 1, figsize=figsize)
         n_conf = corrs.shape[0]
-        colors = sns.husl_palette(n_conf)
+        colors = sns.husl_palette(len(corrs))
 
         for i, (cond, conf_corrs) in enumerate(corrs.iteritems()):
-            barpos = np.linspace(i, i + 1, n_conf + 1)[:-1]
-            bars = ax.bar(barpos, conf_corrs.abs(), width=1 / n_conf,
-                          color=colors, linewidth=0)
+            barpos = np.linspace(i, i + 1, n_conf + 1)[:-1][::-1]
+            bars = ax.barh(barpos, conf_corrs.abs(), height=1 / n_conf,
+                           color=colors, linewidth=.3, edgecolor="white")
 
-        ax.set_xticks(np.arange(len(self._condition_names)) + 0.5)
-        ax.set_xticklabels(self._condition_names)
-        ax.set_xlim(0, len(self._condition_names))
-        ax.xaxis.grid(False)
+        ax.set_xlim(0, 1)
+        ax.set_xlabel("| correlation |")
+        ax.set_yticks(np.arange(len(self._condition_names)) + 0.5)
+        ax.set_yticklabels(self._condition_names)
+        ax.set_ylim(0, len(self._condition_names))
+        ax.yaxis.grid(False)
 
-        ymin, ymax = ax.get_ylim()
-        ymax = max(.25, ymax)
-        ax.set_ylim(0, ymax)
-        ax.set_ylabel("abs(correlation)")
-
-        for x in range(1, len(self._condition_names)):
-            ax.axvline(x, ls=":", c="#222222", lw=1)
+        for y in range(1, len(self._condition_names)):
+            ax.axhline(y, ls=":", c=".6", lw=1, zorder=0)
 
         if legend:
             ncol = len(self._confound_names) // 15 + 1
@@ -490,21 +523,27 @@ class DesignMatrix(object):
             lgd = []
 
         if fname is not None:
-            f.savefig(fname, bbox_extra_artists=[lgd], bbox_inches="tight")
+            f.savefig(fname, dpi=100,
+                      bbox_extra_artists=[lgd],
+                      bbox_inches="tight")
+        if close:
+            plt.close(f)
 
-    def plot_singular_values(self, fname=None):
+    def plot_singular_values(self, fname=None, close=False):
         """Plot the singular values of the full design matrix."""
         s = self._singular_values
         smat = s * np.eye(len(s))
 
         size = min(.3 * len(s), 8)
-        f, ax = plt.subplots(1, 1, figsize=(size, size))
+        f, ax = plt.subplots(figsize=(size, size))
         ax.matshow(smat, cmap="bone", zorder=2)
         ax.axis("off")
 
         plt.tight_layout()
         if fname is not None:
-            f.savefig(fname)
+            f.savefig(fname, dpi=100)
+        if close:
+            plt.close(f)
 
     def to_csv(self, fname):
         """Save the full design matrix to csv."""
